@@ -16,15 +16,15 @@ use crate::{
         Collection, CollectionAuthorityRecord, DataV2, Key, MasterEditionV1, MasterEditionV2,
         Metadata, TokenStandard, UseAuthorityRecord, UseMethod, Uses, BURN, COLLECTION_AUTHORITY,
         COLLECTION_AUTHORITY_RECORD_SIZE, EDITION, MAX_MASTER_EDITION_LEN, PREFIX, USER,
-        USE_AUTHORITY_RECORD_SIZE,
+        USE_AUTHORITY_RECORD_SIZE, EDITION_MARKER_BIT_SIZE
     },
     solana_program::{
         program_memory::{ sol_memset},
     },
     utils::{
         assert_currently_holding, assert_data_valid, assert_derivation, assert_initialized,
-        assert_mint_authority_matches_mint, assert_owned_by, assert_signer,
-        assert_token_program_matches_package, assert_update_authority_is_correct,
+        assert_mint_authority_matches_mint, assert_owned_by, assert_signer, assert_delegated_tokens,
+        assert_token_program_matches_package, assert_update_authority_is_correct, assert_freeze_authority_matches_mint,
         create_or_allocate_account_raw, get_owner_from_token_account,
         process_create_metadata_accounts_logic,
         process_mint_new_edition_from_master_edition_via_token_logic, puff_out_data_fields,
@@ -39,12 +39,12 @@ use solana_program::{
     account_info::{next_account_info, AccountInfo},
     entrypoint::ProgramResult,
     msg,
-    program::invoke,
+    program::{invoke, invoke_signed},
     program_error::ProgramError,
     pubkey::Pubkey,
 };
 use spl_token::{
-    instruction::{approve, revoke},
+    instruction::{approve, revoke, freeze_account, thaw_account},
     state::{Account, Mint},
 };
 use spl_token::instruction::close_account;
@@ -122,6 +122,10 @@ pub fn process_instruction<'a>(
             msg!("Instruction: Sign Metadata");
             process_sign_metadata(program_id, accounts)
         }
+        MetadataInstruction::RemoveCreatorVerification => {
+            msg!("Instruction: Remove Creator Verification");
+            process_remove_creator_verification(program_id, accounts)
+        }
         MetadataInstruction::DeprecatedMintPrintingTokensViaToken(_args) => {
             msg!("Instruction: Deprecated Mint Printing Tokens Via Token, Removed in 1.1.0");
             Err(MetadataError::Removed.into())
@@ -195,6 +199,14 @@ pub fn process_instruction<'a>(
             msg!("Instruction: Revoke Collection Authority");
             process_revoke_collection_authority(program_id, accounts)
         }
+        MetadataInstruction::FreezeDelegatedAccount => {
+            msg!("Instruction: Freeze Delegated Account");
+            process_freeze_delegated_account(program_id, accounts)
+        }
+        MetadataInstruction::ThawDelegatedAccount => {
+            msg!("Instruction: Thaw Delegated Account");
+            process_thaw_delegated_account(program_id, accounts)
+        }
     }
 }
 
@@ -364,6 +376,37 @@ pub fn process_sign_metadata(program_id: &Pubkey, accounts: &[AccountInfo]) -> P
     Ok(())
 }
 
+pub fn process_remove_creator_verification(program_id: &Pubkey, accounts: &[AccountInfo]) -> ProgramResult {
+    let account_info_iter = &mut accounts.iter();
+
+    let metadata_info = next_account_info(account_info_iter)?;
+    let creator_info = next_account_info(account_info_iter)?;
+
+    assert_signer(creator_info)?;
+    assert_owned_by(metadata_info, program_id)?;
+
+    let mut metadata = Metadata::from_account_info(metadata_info)?;
+
+    if let Some(creators) = &mut metadata.data.creators {
+        let mut found = false;
+        for creator in creators {
+            if creator.address == *creator_info.key {
+                creator.verified = false;
+                found = true;
+                break;
+            }
+        }
+        if !found {
+            return Err(MetadataError::CreatorNotFound.into());
+        }
+    } else {
+        return Err(MetadataError::NoCreatorsPresentOnMetadata.into());
+    }
+    metadata.serialize(&mut *metadata_info.try_borrow_mut_data()?)?;
+
+    Ok(())
+}
+
 /// Create master edition
 pub fn process_create_master_edition(
     program_id: &Pubkey,
@@ -1221,3 +1264,123 @@ pub fn set_and_verify_collection(program_id: &Pubkey, accounts: &[AccountInfo])
     metadata.serialize(&mut *metadata_info.try_borrow_mut_data()?)?;
     Ok(())
 }
+
+pub fn process_freeze_delegated_account(
+    program_id: &Pubkey,
+    accounts: &[AccountInfo],
+) -> ProgramResult {
+    let account_info_iter = &mut accounts.iter();
+    let delegate_info = next_account_info(account_info_iter)?;
+    let token_account_info = next_account_info(account_info_iter)?;
+    let edition_info = next_account_info(account_info_iter)?;
+    let mint_info = next_account_info(account_info_iter)?;
+    let token_program_account_info = next_account_info(account_info_iter)?;
+
+    if *token_program_account_info.key != spl_token::id() {
+        return Err(MetadataError::InvalidTokenProgram.into());
+    }
+
+    // assert that edition pda is the freeze authority of this mint 
+    let mint: Mint = assert_initialized(mint_info)?;
+    assert_owned_by(edition_info, program_id)?;
+    assert_freeze_authority_matches_mint(&mint.freeze_authority, edition_info)?;
+
+    // assert delegate is signer and delegated tokens
+    assert_signer(&delegate_info)?;
+    assert_delegated_tokens(
+        delegate_info,
+        mint_info,
+        token_account_info,
+    )?;
+
+
+    let edition_info_path = Vec::from([
+        PREFIX.as_bytes(),
+        program_id.as_ref(),
+        &mint_info.key.as_ref(),
+        EDITION.as_bytes(),
+    ]);
+    let edition_info_path_bump_seed = &[assert_derivation(
+        program_id,
+        edition_info,
+        &edition_info_path,
+    )?];
+    let mut edition_info_seeds = edition_info_path.clone();
+    edition_info_seeds.push(edition_info_path_bump_seed);
+    invoke_signed(
+        &freeze_account(
+            &token_program_account_info.key,
+            &token_account_info.key,
+            &mint_info.key,
+            &edition_info.key,
+            &[],
+        )
+        .unwrap(),
+        &[
+            token_account_info.clone(),
+            mint_info.clone(),
+            edition_info.clone(),
+        ],
+        &[&edition_info_seeds],
+    )?;
+    Ok(())
+}
+
+pub fn process_thaw_delegated_account(
+    program_id: &Pubkey,
+    accounts: &[AccountInfo],
+) -> ProgramResult {
+    let account_info_iter = &mut accounts.iter();
+    let delegate_info = next_account_info(account_info_iter)?;
+    let token_account_info = next_account_info(account_info_iter)?;
+    let edition_info = next_account_info(account_info_iter)?;
+    let mint_info = next_account_info(account_info_iter)?;
+    let token_program_account_info = next_account_info(account_info_iter)?;
+    if *token_program_account_info.key != spl_token::id() {
+        return Err(MetadataError::InvalidTokenProgram.into());
+    }
+
+    // assert that edition pda is the freeze authority of this mint 
+    let mint: Mint = assert_initialized(mint_info)?;
+    assert_owned_by(edition_info, program_id)?;
+    assert_freeze_authority_matches_mint(&mint.freeze_authority, edition_info)?;
+   
+    // assert delegate is signer and delegated tokens
+    assert_signer(&delegate_info)?;
+    assert_delegated_tokens(
+        delegate_info,
+        mint_info,
+        token_account_info,
+    )?;
+    
+    let edition_info_path = Vec::from([
+        PREFIX.as_bytes(),
+        program_id.as_ref(),
+        &mint_info.key.as_ref(),
+        EDITION.as_bytes(),
+    ]);
+    let edition_info_path_bump_seed = &[assert_derivation(
+        program_id,
+        edition_info,
+        &edition_info_path,
+    )?];
+    let mut edition_info_seeds = edition_info_path.clone();
+    edition_info_seeds.push(edition_info_path_bump_seed);
+    invoke_signed(
+        &thaw_account(
+            &token_program_account_info.key,
+            &token_account_info.key,
+            &mint_info.key,
+            &edition_info.key,
+            &[],
+        )
+        .unwrap(),
+        &[
+            token_account_info.clone(),
+            mint_info.clone(),
+            edition_info.clone(),
+        ],
+        &[&edition_info_seeds],
+    )?;
+    Ok(())
+}
